require "savon"

module Intersail
  module Rpdf
    class ZapPdfClient
      cattr_accessor(:max_tries) { 1 }
      cattr_accessor(:min_try_delay) { 1000 }
      cattr_accessor(:max_try_delay) { 5000 }

      def initialize(address)
        @url = address
        @ws = Savon.client(wsdl: "#{address}?wsdl")
      end

      def url
        @url
      end

      def make_pdf_with_local_file(templateFile, data, options = {})
        max_tries = options.delete(:max_tries) || self.class.max_tries

        templateData = Base64.encode64(File.read(templateFile))

        try = 1
        loop do
          response = @ws.call(:make_pdf_with_xml, message: { data: templateData, xmlTags: data.to_xml(options) })

          pdfData = response.nil? ? nil : response.body[:make_pdf_with_xml_response][:make_pdf_with_xml_result]

          return  Base64.decode64(pdfData) unless pdfData.nil?

          if try < max_tries
            try += 1
            sleep delay
          else
            break
          end
        end

        nil
      end

      def get_template_tags(templateFile, options = {})
        max_tries = options.delete(:max_tries) || self.class.max_tries

        templateData = Base64.encode64(File.read(templateFile))

        try = 1
        loop do

          response = @ws.call(:get_template_tags, message: { data: templateData })

          tags = response.nil? ? nil : response.body[:get_template_tags_response][:get_template_tags_result]

          return Intersail::Rpdf::Node.new(tags) unless tags.nil?

          if try < max_tries
            try += 1
            sleep delay
          else
            break
          end
        end

        nil
      end

      def delay
        rand(self.class.min_try_delay...self.class.max_try_delay) / 1000.0
      end
    end
  end
end
